#!/usr/bin/perl -w

##
#   PROMETHEUS (First release)
#   Mac OS X system firewall configuration tool
#
#   Tested on Mac OS X 10.4.8
#   
#   This software has been developped by BlackAngels Staff
#   for AMUG Siena use and distribution.
#   It's possible to modify the present source code, 
#   but it's necessary to keep the original header
#   and informations about author.
#
#   BlackAngels Staff refuse any responsabilities for
#   an improper use of the present software.
#
#   For more informations:
#   http://www.amugsiena.it/index.php?option=com_content&task=view&id=85&Itemid=50
##


# Libraries
use Tie::File;


# Header
$header = "\n\n                                   ,--. ,--.                     \n" .
		  " ,---.,--.--.,---.,--,--,--.,---.,-'  '-|  ,---. ,---.,--.,--.,---.  \n" .
		  "| .-. |  .--| .-. |        | .-. '-.  .-|  .-.  | .-. |  ||  (  .-'  \n" .
		  "| '-' |  |  ' '-' |  |  |  \\   --. |  | |  | |  \\   --'  ''  .-' `)\n" .
		  "|  |-'`--'   `---'`--`--`--'`----' `--' `--' `--'`----'`----'`----'  \n" .
		  "`--'\n" .
		  "                                            BlackAngels Staff \n";


# Constants
$0 = "Prometheus - First Release";
$oldlog = "/var/log/system.log";
$newlog = "/var/log/ipfw.log";
$logconf = "/etc/syslog.conf";
$firstline = "!-ipfw";
$lastline1 = "!ipfw";
$lastline2 = "*.*     /var/log/ipfw.log";
$rulesfile = "/etc/rc.firewall";
$firewalldir = "/Library/StartupItems/Firewall/";
$script1 = $firewalldir . "Firewall";
$script2 = $firewalldir . "StartupParameters.plist";
$hostconf = "/etc/hostconfig";


# PROMETHEUS main
print $header;
if (defined $ARGV[0]) {
if ($ARGV[0] eq "-h") {
	print "\nUsage:";
	print "\n-h	Shows this help";
	print "\n-c	Configures system firewall";
	print "\n-s	Tries to start an already configured firewall";
	print "\n-t	Tries to stop an already running firewall";
	print "\n-r	Tries to restart an already running firewall";
	print "\n-u	Uninstall any file used by a previous configured firewall\n\n";
					  }
elsif ($ARGV[0] eq "-c") { configure(); }
elsif ($ARGV[0] eq "-s") { start(); }
elsif ($ARGV[0] eq "-t") { stop(); }
elsif ($ARGV[0] eq "-r") { restart(); }
elsif ($ARGV[0] eq "-u") { uninstall(); }}
else {
	  MAIN:
	  
	  do {
	  print "\n[*] Permitted operations:";
	  print "\n	1 - Configure a new firewall";
	  print "\n	2 - Start an existing firewall";
	  print "\n	3 - Stop an existing running firewall";
	  print "\n	4 - Restart an existing running firewall";
	  print "\n	5 - Uninstall an existing firewall";
	  print "\n	6 - Exit Prometheus";
	  print "\n\n[*] Choose an option: ";
	  $operation = <STDIN>;
	  chomp($operation);
		} while ($operation != 1 && $operation != 2 && $operation != 3 && $operation != 4 && $operation != 5);

	  print "\n----------------------------------------------\n";

	  if ($operation == 1) { configure(); }
	  elsif ($operation == 2) { start(); }
	  elsif ($operation == 3) { stop(); }
	  elsif ($operation == 4) { restart(); }
	  elsif ($operation == 5) { uninstall(); }
	  elsif ($operation == 6) { exit(1); }
	 }


# Subroutines
sub configure {
print "\n[+] Getting actual firewall rules list\n\n";
system("sudo ipfw list");

print "\n[+] Changing log file from " . $oldlog . " to " . $newlog;
open(SYSLOG, "<$logconf");
@syslog = <SYSLOG>;
close(SYSLOG);
unshift @syslog, $firstline;
push @syslog, $lastline1;
push @syslog, $lastline2;
$loglen = @syslog;
open(SYSLOG, ">$logconf");
for($i=0;$i<$loglen;$i++) { print SYSLOG $syslog[$i] . "\n"; }
close(SYSLOG);

print "\n[+] Restarting syslogd to load new configuration";
system("sudo kill -s HUP `cat /var/run/syslog.pid`");

print "\n[+] Turning logging on\n\n";
system("sudo sysctl -w net.inet.ip.fw.verbose=1");

CREATERULES:

do {

	print "\n[+] Firewall rules creation";
	print "\n	[1] Step by step configuration";
	print "\n	[2] Manual configuration";
	print "\n\n\tPlease choose an option (Default: 1): ";
	$configtype = <STDIN>;
	chomp($configtype);
		
   } while ($configtype ne 1 && $configtype ne 2 && $configtype ne "");
   
if ($configtype eq 1 || $configtype eq "") {

	print "\n\t  [ Step by step configuration, will ask you for the type of service to allow," .
		  "\n\t    or to deny and then it will automatically create correspondent rules." .
		  "\n\t    It will also suggest you the optimal settings in some cases." .
		  "\n\t    Use \"y\" for YES and \"n\" for NO. ]";
		  
	open(RULES, ">$rulesfile");
	print RULES "\#!/bin/sh\n";
	do {
		print "\n\n\t[-] Allow localhost loopback (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { print RULES "ipfw add allow ip from any to any via lo*\n"; }
	
	do {
		print "\t[-] Deny localhost spoofing (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { 
										  print RULES "ipfw add deny ip from 127.0.0.0/8 to any in\n";
										  print RULES "ipfw add deny ip from any to 127.0.0.0/8 in\n";
										 }
										 
	do {
		print "\t[-] Deny other common spoofing (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { 
										  print RULES "ipfw add deny ip from 224.0.0.0/3 to any in\n";
										  print RULES "ipfw add deny tcp from any to 224.0.0.0/3 in\n";
										 }
										 
	do {
		print "\t[-] Allow outgoing traffic (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { print RULES "ipfw add allow tcp from any to any out\n"; }
	
	do {
		print "\t[-] Allow estabilished connections (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { print RULES "ipfw add allow tcp from any to any established\n"; }
	
	do {
		print "\t[-] Deny Microsoft Office PID detection: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add deny udp from any to any 2222\n"; }
	
	do {
		print "\t[-] Allow Remote Apple Events: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 3031 in\n"; }	
	
	do {
		print "\t[-] Allow Remote Login (SSH): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 22 in\n"; }
	
	do {
		print "\t[-] Allow Windows Sharing (Samba): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 139 in\n"; }
	
	do {
		print "\t[-] Allow iTunes Music Sharing: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 3689 in\n"; }
	
	do {
		print "\t[-] Allow CVS: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 2401 in\n"; }
	
	do {
		print "\t[-] Allow Gnutella/Limewire: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 6346 in\n"; }
	
	do {
		print "\t[-] Allow ICQ: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 4000 in\n"; }
	
	do {
		print "\t[-] Allow IRC: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 194 in\n"; }
	
	do {
		print "\t[-] Allow MSN Messenger: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 6891-6900 in\n"; }
	
	do {
		print "\t[-] Allow Network Time: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { print RULES "ipfw add allow tcp from any to any 123 in\n"; }
	
	do {
		print "\t[-] Allow Personal File Sharing: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { 
						 print RULES "ipfw add allow tcp from any to any 548 in\n";
						 print RULES "ipfw add allow tcp from any to any 427 in\n";
						 print RULES "ipfw add allow tcp from any to any 80 in\n";
						}
						
	do {
		print "\t[-] Allow FTP Access: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { 
						 print RULES "ipfw add allow tcp from any to any 20-21 in\n";
						 print RULES "ipfw add allow tcp from any 20,21 to any 1024-65535 in\n";
						}
						
	do {
		print "\t[-] Allow iChat Rendezvous: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { 
						 print RULES "ipfw add allow tcp from any to any 5297 in\n";
						 print RULES "ipfw add allow tcp from any to any 5298 in\n";
						}
						
	do {
		print "\t[-] Allow Printer Sharing: ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n");
	if ($answer eq "y") { 
						 print RULES "ipfw add allow tcp from any to any 631 in\n";
						 print RULES "ipfw add allow tcp from any to any 515 in\n";
						}
						
	do {
		print "\t[-] Deny all other unspecified TCP traffic (Default: y): ";
		$answer = <STDIN>;
		chomp($answer);
	   } while ($answer ne "y" && $answer ne "n" && $answer ne "");
	if ($answer eq "y" || $answer eq "") { print RULES "ipfw add deny tcp from any to any\n"; }
	
	close(RULES);
		  
										   }
elsif ($configtype == 2) {

	   print "\n\t  [ Manual configuration will try to use input data, to create custom firewall" .
			 "\n\t    rules, by a series of requests to the user." .
			 "\n\t    The string format used for IPFW rules is:" .
			 "\n\t    [action] [protocol] from [source] to [destination] [interface(optional)]" .
			 "\n\t    Type \"__EOF__\" to set the end of rules file ]";
			 
	   do {
		   print "\n\n\t[-] This will clear actual rules file \"" . $rulesfile . "\". Continue (Default: n): ";
		   $clear =<STDIN>;
		   chomp($clear);
		  } while ($clear ne "y" && $clear ne "n" && $clear ne "");
	   if ($clear eq "y") {
	   open(RULES, ">$rulesfile");
	   print RULES "\#!/bin/sh\n";

	   while ($ruleline = <STDIN>) {
			  chomp($ruleline);
			  if ($ruleline eq "__EOF__") { last; }
			  elsif ($ruleline =~ /([^\s]+?) ([^\s]+?) from ([^\s]+?) to ([^\s]+?)/) {
					 if ($ruleline eq "ipfw add deny tcp from any to any") {
						 do {
							 print "\n\n\t[-] This must be the last line of rules file. Are you sure (Default: y): ";
							 $last = <STDIN>;
							 chomp($last);
						    } while ($last ne "y" && $last ne "n" && $last ne "");
						 if ($last eq "n") { next; }
																		   }
					 print RULES "ipfw add " . $ruleline . "\n";
					 last;
																					 }
			  else { 
					print "\n\n\t[-] Invalid rule line format. Rules must be in format:";
					print "\n\t    [action] [protocol] from [source] to [destination] [interface(optional)]";
					next; 
				   }
								   }

	   close RULES;
						  }
	   else { goto CREATERULES; }
		  
						 }
						 
print "\n[+] Firewall rules file successfully created";
print "\n[+] Creating firewall directory \"" . $firewalldir . "\"";
system("sudo mkdir -p " . $firewalldir);

print "\n[+] Creating firewall main script \"" . $script1 . "\"";
open(SCRIPT, ">$script1");
print SCRIPT "\#!/bin/sh\n\n";
print SCRIPT "\#\#\n";
print SCRIPT "\# Firewall\n";
print SCRIPT "\#\#\n\n";
print SCRIPT ". /etc/rc.common\n\n";
print SCRIPT "StartService ()\n";
print SCRIPT "{\n";
print SCRIPT "  if [ \"\${FIREWALL:=-NO-}\" = \"-YES-\" ]\n";
print SCRIPT "  then\n";
print SCRIPT "    sh /etc/rc.firewall > /dev/null\n";
print SCRIPT "  fi\n";
print SCRIPT "}\n\n";
print SCRIPT "StopService ()\n";
print SCRIPT "{\n";
print SCRIPT "  /sbin/ipfw -f -q flush\n";
print SCRIPT "}\n\n";
print SCRIPT "RestartService ()\n";
print SCRIPT "{\n";
print SCRIPT "  StopService\n";
print SCRIPT "  StartService\n";
print SCRIPT "}\n\n";
print SCRIPT "RunService \"\$1\"\n";
close(SCRIPT);

print "\n[+] Creating firewall startup script \"" . $script2 . "\"";
open(SCRIPT, ">$script2");
print SCRIPT "{\n";
print SCRIPT "  Description     = \"Firewall\";\n";
print SCRIPT "  Provides        = (\"Firewall\");\n";
print SCRIPT "  Requires        = (\"NetworkExtensions\",\"Resolver\");\n";
print SCRIPT "  OrderPreference = \"Late\";\n";
print SCRIPT "  Messages =\n";
print SCRIPT "  {\n";
print SCRIPT "    start = \"Starting firewall\";\n";
print SCRIPT "    stop  = \"Stopping firewall\";\n";
print SCRIPT "  };\n";
print SCRIPT "}\n";
close(SCRIPT);

print "\n[+] Setting firewall automatically load at startup";
open(HOSTCONF,">>$hostconf");
print HOSTCONF "FIREWALL=-YES-\n";
close(HOSTCONF);

print "\n[+] Changing files owners, groups and permissions\n";
print "\n	[-] \"" . $script1 . "\" owner -> root";
system("sudo chown -R root  /Library/StartupItems/Firewall");
print "\n	[-] \"" . $script1 . "\" group -> wheel";
system("sudo chgrp -R wheel /Library/StartupItems/Firewall");
print "\n	[-] \"" . $script1 . "\" permissions -> 755";
system("sudo chmod -R 755 /Library/StartupItems/Firewall");
print "\n	[-] \"" . $rulesfile . "\" owner -> root";
system("sudo chown root /etc/rc.firewall");
print "\n	[-] \"" . $rulesfile . "\" group -> admin";
system("sudo chgrp admin /etc/rc.firewall");
print "\n	[-] \"" . $rulesfile . "\" permissions -> 755";
system("sudo chmod 755 /etc/rc.firewall");

print "\n\n[+] System firewall successfully configured";
do {
	print "\n[+] Would you like to start firewall now [y,n] (Default: y): ";
	$start = <STDIN>;
	chomp($start);
   } while ($start ne "y" && $start ne "n" && $start ne "");
if ($start eq "y" || $start eq "") {
	print "\n\n	[-] Starting system firewall";
	system("sudo /Library/StartupItems/Firewall/Firewall start");
	print "	[-] Firewall successfully started";
								   }
print "\n\n[+] Getting updated firewall rules list\n\n";
system("sudo ipfw list");
print "\n[+] Prometheus ready for a new operation ...\n\n";
}

sub start {
print "\n[+] Searching for firewall scripts";
if (-e $rulesfile && -e $script1) {
	print "\n\n	[-] Configured firewall has been found";
	print "\n\n[+] Trying to start system firewall";
	system("sudo /Library/StartupItems/Firewall/Firewall start");
	print "\n	[-] Firewall successfully started";
	print "\n\n[+] Prometheus ready for a new operation ...\n\n";
								  }
else {
	  print "\n\n	[-] Unable to find configured firewall";
	  print "\n\n[+] Prometheus ready for a new operation ...\n\n";
	 }
goto MAIN;
}

sub stop {
print "\n[+] Searching for firewall scripts";
if (-e $rulesfile && -e $script1) {
	print "\n\n	[-] Configured firewall has been found";
	print "\n\n[+] Trying to stop system firewall";
	system("sudo /Library/StartupItems/Firewall/Firewall stop");
	print "\n	[-] Firewall successfully stopped";
	print "\n\n[+] Prometheus ready for a new operation ...\n\n";
								  }
else {
	  print "\n\n	[-] Unable to find configured firewall";
	  print "\n\n[+] Prometheus ready for a new operation ...\n\n";
	 }
goto MAIN;
}

sub restart {
print "\n[+] Searching for firewall scripts";
if (-e $rulesfile && -e $script1) {
	print "\n\n	[-] Configured firewall has been found";
	print "\n\n[+] Trying to restart system firewall";
	system("sudo /Library/StartupItems/Firewall/Firewall restart");
	print "\n	[-] Firewall successfully restarted";
	print "\n\n[+] Prometheus ready for a new operation ...\n\n";
								  }
else {
	  print "\n\n	[-] Unable to find configured firewall";
	  print "\n\n[+] Prometheus ready for a new operation ...\n\n";
	 }
goto MAIN;
}

sub uninstall {
print "\n[+] Searching for firewall scripts";
if (-e $rulesfile || -e $script1 || -e $script1) {
	print "\n\n	[-] Configured firewall has been found";
	print "\n\n[+] Trying to remove system firewall scripts and directory";
	system("sudo rm " . $rulesfile);
	print "\n\n	[-] \"" . $rulesfile . "\" -> REMOVED";
	system("sudo rm " . $script1);
	print "\n	[-] \"" . $script1 . "\" -> REMOVED";
	system("sudo rm " . $script2);
	print "\n	[-] \"" . $script2 . "\" -> REMOVED";
	system("sudo rmdir " . $firewalldir);
	print "\n	[-] \"" . $firewalldir . "\" -> REMOVED";
	print "\n\n[+] Turning logging off\n\n";
	system("sudo sysctl -w net.inet.ip.fw.verbose=0");
	tie @logconfig, 'Tie::File', $logconf;
	for (@logconfig) { 
					  s{\Q$firstline\E}{}g;
					  s{\Q$lastline1\E}{}g;
					  s{\Q$lastline2\E}{}g;
					 }
	untie @logconfig;
	print "\n[+] Removing automatical firewall load on startup";
	tie @hostconfig, 'Tie::File', $hostconf;
	for (@hostconfig) { s/FIREWALL=-YES-//g; }
	untie @hostconfig;
	print "\n\n	[-] Uninstall successfully completed";
	print "\n\n[+] Prometheus ready for a new operation ...\n\n";
												}
else {
	  print "\n\n	[-] Unable to find configured firewall";
	  print "\n\n[+] Prometheus ready for a new operation ...\n\n";
	 }
goto MAIN;
}
